@@ -341,16 +341,16 @@ class TestMediaIoBaseUpload(unittest.TestCase):
     upload = MediaIoBaseUpload(
         fd=fd, mimetype='image/png', chunksize=500, resumable=True)
 
-    # Simulate 5XXs for both the request that creates the resumable upload and
-    # the upload itself.
+    # Simulate errors for both the request that creates the resumable upload
+    # and the upload itself.
     http = HttpMockSequence([
       ({'status': '500'}, ''),
       ({'status': '500'}, ''),
       ({'status': '503'}, ''),
       ({'status': '200', 'location': 'location'}, ''),
-      ({'status': '500'}, ''),
-      ({'status': '500'}, ''),
-      ({'status': '503'}, ''),
+      ({'status': '403'}, USER_RATE_LIMIT_EXCEEDED_RESPONSE),
+      ({'status': '403'}, RATE_LIMIT_EXCEEDED_RESPONSE),
+      ({'status': '429'}, ''),
       ({'status': '200'}, '{}'),
     ])
 
@@ -372,6 +372,34 @@ class TestMediaIoBaseUpload(unittest.TestCase):
     request.execute(num_retries=3)
     self.assertEqual([20, 40, 80, 20, 40, 80], sleeptimes)
 
+  def test_media_io_base_next_chunk_no_retry_403_not_configured(self):
+    fd = BytesIO(b"i am png")
+    upload = MediaIoBaseUpload(
+        fd=fd, mimetype='image/png', chunksize=500, resumable=True)
+
+    http = HttpMockSequence([
+        ({'status': '403'}, NOT_CONFIGURED_RESPONSE),
+        ({'status': '200'}, '{}')
+        ])
+
+    model = JsonModel()
+    uri = u'https://www.googleapis.com/someapi/v1/upload/?foo=bar'
+    method = u'POST'
+    request = HttpRequest(
+        http,
+        model.response,
+        uri,
+        method=method,
+        headers={},
+        resumable=upload)
+
+    request._rand = lambda: 1.0
+    request._sleep =  mock.MagicMock()
+
+    with self.assertRaises(HttpError):
+      request.execute(num_retries=3)
+    request._sleep.assert_not_called()
+
 
 class TestMediaIoBaseDownload(unittest.TestCase):
 
@@ -1072,6 +1100,34 @@ class TestBatch(unittest.TestCase):
       header = parts[1].splitlines()[1]
       self.assertEqual('Content-Type: application/http', header)
 
+  def test_execute_initial_refresh_oauth2(self):
+    batch = BatchHttpRequest()
+    callbacks = Callbacks()
+    cred = MockCredentials('Foo')
+
+    # Pretend this is a OAuth2Credentials object
+    cred.access_token = None
+
+    http = HttpMockSequence([
+      ({'status': '200',
+        'content-type': 'multipart/mixed; boundary="batch_foobarbaz"'},
+       BATCH_SINGLE_RESPONSE),
+    ])
+
+    cred.authorize(http)
+
+    batch.add(self.request1, callback=callbacks.f)
+    batch.execute(http=http)
+
+    self.assertEqual({'foo': 42}, callbacks.responses['1'])
+    self.assertIsNone(callbacks.exceptions['1'])
+
+    self.assertEqual(1, cred._refreshed)
+
+    self.assertEqual(1, cred._authorized)
+
+    self.assertEqual(1, cred._applied)
+
   def test_execute_refresh_and_retry_on_401(self):
     batch = BatchHttpRequest()
     callbacks = Callbacks()
